/**
 * Core utilities and functions.
 */
Ext.define('App', {

    singleton: true,
    uses: ['Ext.data.ArrayStore',
           'Ext.data.DirectStore',
           'Ext.data.JsonPStore',
           'Ext.data.JsonStore',
           'App.data.DirectStore',
           'Ext.data.XmlStore'],

    /**
     * The version of the plugin
     * @property {String}
     */
    version: '0.6.2.5',

    /**
     * The name of the application
     * @property {String}
     */
    name: undefined,

    /**
     * The configuration parameters of the application.
     *
     * @property {Object} config
     * @property {Object} config.models
     * The definition of the model associated to each database table.
     *
     * @property {Object} config.stores
     * The configuration of the store associated to each database table.
     *
     * @property {Array} config.treeNodes
     * The list of nodes for the tree store.
     *
     * @property {Object} config.viewport
     * The configuration of the viewport.
     *
     */
    config: null,

    /**
     * @property {String}
     * The URL of the controller/action used to produce CSV file,
     * namely /application/plugin_dbui/csv.
     */
    csvUrl: undefined,

    /**
     * @property {Boolean}
     * Activate the debug mode on the client side.
     * In this mode the Ext JS an the application classes
     * are loaded dynamically.
     */
    debug: false,

    /**
     * @property {String}
     * The URL of the controller to convert latex string into PDF file,
     * namely /application/plugin_dbui/latex2pdf.pdf.
     */
    latex2pdfUrl: undefined,

    /**
     * The configuration of the Ext.Direct service.
     * It contains the definition of the remote procedure.
     * @property {Object} REMOTE_API
     * @property {String} REMOTE_API.url
     * The well-formed URL for the call method running on the server side:
     *
     *     /myapplication/plugin_dbui/call
     *
     * @property {String} REMOTE_API.type="remoting"
     *
     * @property {Object} REMOTE_API.actions
     * The registered actions are encoded:
     *
     *     Dbui.methodName
     *
     * Each action has the following properties:
     * @property {String} REMOTE_API.actions.method
     * @property {Number} REMOTE_API.actions.len
     */
    REMOTE_API: null,

    /**
     * Helper method to build URI string.
     *
     * The URI is built from the arguments and the data are HTML
     * encoded to preserve special character like quote, /, ?, ...
     *
     * @param {String} minetype
     * The mine type of the return file, e.g "text/csv".
     *
     * @param {String} encoding
     * The data encoding, e.g. "utf-8".
     *
     * @param {String/String[]} data
     * The content of the file. Either a string of a list of string,
     * one string per line.
     *
     * @return {String}
     *
     */
    buildURI: function  (minetype, encoding, data) {

        "use strict";

        var uri;

        // build the data string in which special
        // character like quote, / , ? are HTML encoded
        if (Ext.isArray(data)) {
            data = data.join("\n");
        }
        data = encodeURIComponent(data);

        // build the URI
        uri = "data:" + minetype + ";";
        uri += "charset=" + encoding +",";
        uri += data;

        return uri;
    },

    /**
     * Close the user session on the server.
     * Can be triggered by the window event *beforeunload*.
     *
     * @param {Ext.EventObject} event
     */
    closeUserSession: function (event) {
        "use strict";
        event.returnValue = undefined;
        Dbui.close_user_session();
    },

    /**
     * Decode a string in CapitalizeWords into a list of words
     * where all words are in lower case.
     *
     * @param {String} text
     * The field encoded as a CapitalizeWord
     *
     * @return {Array}
     * One entry per word. Each word is in lower casse.
     *
     */
    decodeCapitalizeWords: function (text) {

        "use strict";

        var c,
            i,
            isUpper = /[A-Z]/,
            li = [];

        for (i = 0; i < text.length; i += 1) {

            c = text[i];

            if (c.match(isUpper)) {
                li.push(c.toLowerCase());

            } else {
                li[li.length - 1] += c;
            }
        }

        return li;
    },

    /**
     * Define the model for all database tables.
     */
    defineModels: function () {

        "use strict";

        var modelName;

        for (modelName in this.config.models) {
            if (this.config.models.hasOwnProperty(modelName)) {
                Ext.define(modelName, this.config.models[modelName]);
            }
        }
    },

    /**
     * Mimic the encode_field function running on the server.
     * It encodes the pair (table, field) as TableField.
     *
     * @param {String} table
     * @param {String} field
     *
     * @return {String} The input arguments encoded as TableField.
     */
    encodeField: function (table, field) {
        "use strict";
        var t, f;
        t = table[0].toUpperCase() + table.slice(1, table.length);
        f = field[0].toUpperCase() + field.slice(1, field.length);
        return t + f;
    },

    /**
     * Return the Ext.data.Store identifies by its id.
     * If the store does not exit it is created and register in the store manager.
     *
     * **Note:** The storeId is defined by the server.
     * The syntax of the storeId is *nameStore* where *name* corresponds
     * to a database table.
     * The store configuration is extract from the array App#config.stores.
     *
     * In the current framework, this function associates a unique store
     * to a database table. Therefore it can be shared by different widgets.
     *
     * @param {String} storeId
     *
     * @return {Ext.data.Store}
     */
    getStore: function (storeId) {

        "use strict";

        var cfg,
            className,
            store;

        // return an existing store
        store = Ext.StoreMgr.lookup(storeId);

        // the store already exists
        if (store) {
            return store;
        }

        // protection
        if (!store && !Ext.isString(storeId)) {
            throw new Error('Fail to instanciate the store: "' + storeId + '"');
        }

        // get the configuration
        cfg = this.config.stores[storeId];

        // instantiate the store
        switch (cfg.xtype) {

        case 'array':
            className = 'Ext.data.ArrayStore';
            break;

        case 'direct':
            className = 'Ext.data.DirectStore';
            break;

        case 'jsonpstore':
            className = 'Ext.data.JsonPStore';
            break;

        case 'json':
            className = 'Ext.data.JsonStore';
            break;

        case 'xdirectstore':
            className = 'App.data.DirectStore';
            break;

        case 'xml':
            className = 'Ext.data.XmlStore';
            break;

        default:
            throw new Error('Fail to instanciate the "' + cfg.xtype + '" store !');
        }
        return Ext.create(className, cfg);
    },

    /**
     * Return the name of the database table associated to the store.
     *
     * @param {App.data.DirectStore} store
     *
     * @return {String}
     */
    getTableName: function (store) {
        "use strict";
        return store.extraParams.tableName;
    },

    /**
     * Determine if a plugin is present in the plugins list of a component.
     *
     * @param {Ext.Component} component
     * @param {String} ptype The ptype of the plugin.
     *
     * @return {Boolean}
     */
    isPlugin: function (component, ptype) {

        "use strict";

        var i,
            plugin;

        if (!component.hasOwnProperty('plugins')) {
            return false;
        }

        if (component.plugins === undefined) {
            return false;
        }

        for (i = 0; i < component.plugins.length; i += 1) {

            plugin = component.plugins[i];

            if (plugin === ptype) {
                return true;
            }

            if ((typeof plugin === 'object') && (plugin.ptype === ptype)) {
                return true;
            }
        }
        return false;
    },

    /**
     * Save the local data as a file using URI technique.
     * The file name is defined by the browser.
     *
     * The URI is built from the arguments and the data are HTML
     * encoded to preserve special character like quote, /, ?, ...
     *
     *
     * @param {String} minetype
     * The mine type of the return file, e.g "text/csv".
     *
     * @param {String} encoding
     * The data encoding, e.g. "utf-8".
     *
     * @param {String/String[]} data
     * The content of the file. Either a string of a list of string,
     * one string per line.
     *
     */
    save: function (minetype, encoding, data) {

        "use strict";
        window.location.href = this.buildURI(minetype, encoding, data);
    },

    /**
     * Save the local data as a file using URI technique.
     * Similar to the #save method but the file name is defined by the user.
     *
     * The URI is built from the arguments and the data are HTML
     * encoded to preserve special character like quote, /, ?, ...
     *
     * @param {String} minetype
     * The mine type of the return file, e.g "text/csv".
     *
     * @param {String} encoding
     * The data encoding, e.g. "utf-8".
     *
     * @param {String/String[]} data
     * The content of the file. Either a string of a list of string,
     * one string per line.
     *
     * @param {String} filename
     * The name of the file.
     */
    saveAs: function (minetype, encoding, data, filename) {

        "use strict";

        var link;

        // download the file by simulating a click on a link
        link = Ext.getBody().createChild({
            tag: "a",
            href: this.buildURI(minetype, encoding, data),
            download: filename
        });

        link.dom.click();
        link.destroy();
    },

    /**
     * activate the dynamic loading when flag is true
     *
     * @param {Boolean} flag
     *
     */
    setDynamicLoading: function (flag) {

        "use strict";

        if (flag) {
            Ext.Loader.setConfig({
                disableCaching: false,
                enabled: true,
                paths: {
                    'App': '../static/plugin_dbui/src',
                    'Ext': '../static/plugin_extjs/src'
                }
            });
        }
    }

});