/**
 * The form panel is an Ext.form.Panel linked to a database table.
 *
 * It contains field widget for the database table fields and
 * two buttons *Action* and *Reset*.
 *
 * The store attribute is equal to the table name when the widget
 * is configured. It is replaced by an App.data.DirectStore
 * in the initialization phase of the component.
 *
 * It also comes with a set of methods to
 * create, duplicate, destroy and update record in the database table.
 * Therefore this component can be used as a classic form, a row editor
 * for a grid, a record browser, ....
 *
 * The action associated to the button is set via the #setAction method.
 *
 */
Ext.define('App.form.Panel', {

    extend: 'Ext.form.Panel',
    alias: 'widget.xform',
    requires: [
        'App.form.plugin.RegExp',
        'App.form.plugin.ToolTip'
    ],
    uses: [
        'App.form.field.AceEditor',
        'App.form.field.ComboBox',
        'App.form.field.ComboBoxMaster',
        'App.form.field.ComboBoxSlave',
        'App.form.field.ComboBoxUserReset',
        'App.form.field.Dict',
        'App.form.field.DictMultiDefault',
        'App.form.field.List',
        'App.form.field.TextPicker',
        'Ext.form.field.Date',
        'Ext.form.field.HtmlEditor'
    ],
    /**
     * @property {Ext.Button}
     * The action button of the form
     */
    buttonAction: null,

    /**
     * @property {Ext.Button}
     * The reset button of the form
     */
    buttonReset: null,

    /**
     * @cfg {Ext.data.Store} store (required)
     * The Ext.data.Store that the form should use.
     * The store has to be defined since the actions rely on it.
     */
    store: null,

    // Predefined configuration options
    autoScroll: true,
    bodyStyle: 'padding:5px 5px 0',
    buttons: [{
        formBind: true,
        itemId: 'buttonAction',
        text: 'Action'
    }, {
        itemId: 'buttonReset',
        text: 'Reset'
    }],
    defaults: {anchor: '100%'},
    defaultType: 'textfield',
    frame: true,

    // private properties to keep track of current action parameters
    currentAction: null,
    currentRecord: null,

    // keep track of the index of the selected record in the store
    // useful to add a new record before or after the selected one
    currentIndex: undefined,

    // private properties for internationalization
    textCreate: 'Create',
    textDestroy: 'Delete',
    textDuplicate: 'Duplicate',
    textReset: 'Reset',
    textUpdate: 'Update',

    textCreateErrorAction: 'The new record(s) is destroyed.',
    textCreateFailed: 'Create failed...',
    textDbAction: 'The database is not modified.',
    textDeleteErrorAction: 'The record(s) is kept.',
    textDeleteFailed: 'Delete failed...',
    textError: 'Error...',
    textServerError: 'Internal server error.',
    textStoreError: 'the store is undefined !!!',
    textUpdateErrorAction: 'The update record(s) is revert to its original values.',
    textUpdateFailed: 'Update failed...',

    // private method require by the ExtJs component model
    initComponent: function () {

        var me = this,
            toolbar;

        // initialise the base class
        me.callParent(arguments);

        // short cuts
        toolbar = me.getDockedItems()[0];
        me.buttonAction = toolbar.getComponent('buttonAction');
        me.buttonReset = toolbar.getComponent('buttonReset');

        // button events listeners
        me.buttonAction.on('click', me.doAction, me);
        me.buttonReset.on('click', me.softReset, me);

        // link the form to the data store
        me.store = App.getStore(me.store);
        me.store.getProxy().on('exception', me.onProxyException, me);
        me.store.on('write', me.softReset, me);

        // set the default action: create
        me.setAction('create');
        me.buttonReset.setText(me.textReset);
    },

    /**
     * Add a new record. It is inserted after the selected one
     * when the attribute #currentIndex is defined.
     *
     * @param {Ext.data.Model} record
     */
    addRecord: function (record) {

        "use strict";

        var me = this;

        // add the new record after the selected one
        if (me.currentIndex !== undefined) {
            me.store.insert(me.currentIndex + 1, record);

        // add the new record at the end of the store
        } else {
            me.store.add(record);
        }

        // reset the index
        me.currentIndex = undefined;
    },

    // private method required by the ExtJS component model
    beforeDestroy: function () {

        var me = this;

        me.buttonAction.un('click', me.doAction, me);
        me.buttonReset.un('click', me.softReset, me);

        me.store.getProxy().un('exception', me.onProxyException, me);
        me.store.un('write', me.softReset, me);

        me.callParent(arguments);
    },

    /**
     * Enable or disable all fields of the form
     * @param {Object} boolean
     */
    disableFields: function (bool) {

        "use strict";

        var me = this,
            fields = me.getForm().getFields();

        fields.each(function (field) {
            field.setDisabled(bool);
            return true;
        });

    },

    /**
     * Execute the #currentAction on the store.
     * The current action is set by the #setAction method.
     *
     * **Note:** Raise an error when the store is not defined.
     */
    doAction: function () {

        "use strict";

        var me = this,
            form = me.getForm(),
            model = me.store.getProxy().getModel(),
            newRecord;

        // No action on non-valid form except destroy
        if (me.currantAction !== 'destroy' && !form.isValid()) {
            return;
        }

        if (!me.store) {
            throw new Error(me.textStoreError);
        }

        switch (me.currentAction) {

        case 'create':
            newRecord = model.create();
            me.updateRecord(newRecord);
            me.addRecord(newRecord);
            break;

        case 'destroy':
            me.store.remove(me.currentRecord);
            break;

        case 'duplicate':
            newRecord = model.create();
            me.updateRecord(newRecord);
            me.addRecord(newRecord);
            break;

        case 'update':
            me.currentRecord.beginEdit();
            me.updateRecord(me.currentRecord);
            me.currentRecord.endEdit();
            break;
        }

        if (!me.store.autoSync) {
            me.store.sync();
        }
    },

    /**
     * Process the proxy exception.
     * See Ext.data.proxy.Direct for the description of the arguments.
     *
     * @param {Ext.data.proxy.Proxy} proxy
     * @param {Object} response
     * @param {Ext.data.Operation} operation
     * @param {Object} eOpts
     *
     */
    onProxyException: function (proxy, response, operation, eOpts) {

        "use strict";

        var me = this,
            errors,
            key,
            msg = "";

        // invalid response from the server, HTPP 400, 500
        if (response instanceof Ext.direct.ExceptionEvent) {
            Ext.Msg.alert(me.textError, response.xhr.responseText);
            return;
        }

        // valid answer from the server, HTTP 200
        // but something went wrong in the server validation process, ...
        if (response instanceof Ext.direct.RemotingEvent) {

            if (!response.result.success) {
                errors = response.result.errors;

                if (Ext.isString(errors)) {
                    msg = errors;

                } else {
                    for (key in errors) {
                        if (errors.hasOwnProperty(key) && errors[key]) {

                            msg += errors[key];
                            msg += "<br>";
                        }
                    }
                }

                Ext.Msg.alert(me.textError, msg);
                return;
            }
        }
    },

    /**
     * Set the action associated to the form.
     * The record is load into the form and the text of the button is
     * properly set.
     *
     * @param {String} action
     * Valid action are create, destroy, duplicate, update and view.
     * @param {Ext.data.Record} record
     */
    setAction: function (action, record) {

        "use strict";

        var me = this,
            form = me.getForm(),
            newRecord,
            table,
            tableId;

        me.buttonReset.show();
        me.buttonAction.show();

        me.disableFields(false);

        me.currentAction = action;
        me.currentRecord = record;

        // force orginalValues to be equal to the load data
        // this is required by the softReset method.
        form.trackResetOnLoad = true;

        switch (action) {

        case 'create':
            me.buttonAction.setText(me.textCreate);

            // load an empty record filled with default values
            record = me.store.getProxy().getModel().create();
            form.loadRecord(record);
            break;

        case 'destroy':
            me.buttonAction.setText(me.textDestroy);
            form.loadRecord(record);
            break;

        case 'duplicate':
            me.buttonAction.setText(me.textDuplicate);

            // create a copy without keeping the reference to internal objects
            newRecord = record.copy();
            Ext.Array.each(newRecord.fields.items, function (field) {
               var value = newRecord.data[field.name];
               if (typeof value === "object") {
                   newRecord.data[field.name] = Ext.clone(value);
               }
            });

            form.loadRecord(newRecord);
            break;

        case 'update':
            me.buttonAction.setText(me.textUpdate);
            form.loadRecord(record);
            break;

        case 'view':
            me.buttonReset.hide();
            me.buttonAction.hide();
            form.loadRecord(record);
            me.disableFields(true);
            break;
        }
    },

    /**
     * Soft reset of the form.
     * The field values are reset to those originally loaded.
     */
    softReset : function () {

        "use strict";
        var me = this;

        me.getForm().reset();
    },

    /**
     * Update the selected record with the value of the form.
     * @param {Ext.data.Model} record
     */
    updateRecord: function (record) {

        "use strict";

        var me = this,
            combo,
            containers,
            fields,
            rec,
            value;

        //
        // get the list of basic fields:
        // those at the first level and those encapsulated in fieldset
        //
        // WARNING: do not use query(field) since it find all fields
        // including those encapsulated in complexe field container
        // which encapsulated grid with cell editing...
        //
        fields = me.query('> field');
        fields = Ext.Array.merge(fields, me.query('fieldset > field'));


        // get the list of field container:
        // those at the first level and those encapsulated in fieldset
        containers = me.query('> fieldcontainer');
        containers = Ext.Array.merge(containers, me.query('fieldset > fieldcontainer'));

        // container handles two type of object:
        //  - composite field merging several standard field
        //  - single field encapsulating complex widget grid with cell editing.
        //
        // the extraction the sub field structure depends on the type
        Ext.Array.each(containers, function(el) {

            // standard field container merging basic fields:
            // retrieve all basic fields
            if (el.getXType() === 'fieldcontainer') {
                Ext.Array.each(el.query('> field'), function(field) {
                    fields.push(field);
                });

            // field container encapsulating complex object:
            // treat it as a single field
            } else {
                fields.push(el);
            }
        });

        // update the record
        // take care of special treatment required by date and combobox
        Ext.Array.each(fields, function(field) {

            value = field.getValue();

            switch (field.getXType()) {

            // We convert the date object according to a string defined
            // by the Ext.form.field.Date property format.
            // NOTE: by default in the json encoding, the date object
            // is converted as string using iso format YYYY-MM-DDTHH:MM:SS.
            // However, the string expected by the database depends on
            // the date type: date, datetime or time. The format of the
            // Ext.form.field.Date, used by the interface, is defined in the
            // property format.

            case 'datefield':
                if (Ext.isDate(value)) {
                    value = Ext.util.Format.date(value, field.format);
                }
                break;

            // For foreign key, the record contains the valueField
            // as well as the displayField. The default action update
            // the valueField but note the display field.
            // The next lines append the displayField

            case 'xcombobox':
                combo = field;
                rec = combo.findRecord(combo.valueField, combo.getValue());
                record.set(combo.displayField, rec.get(combo.displayField));
                break;
            }

            record.set(field.getName(), value);
        });
    }
});
