/**
 * The App.form.field.ComboBoxMaster and the App.form.field.ComboBoxSlave
 * are two standard Ext.form.field.ComboBox. They are linked together in
 * such a way that the values available in the slave depends on the
 * selected value in the master.
 *
 * Both are bind to the unique store, the
 * {@link App.form.field.ComboBoxMaster#refStore refStore}.
 * It defines the relation between master and slave values. It has to contain the
 * displayField and the valueField for the master and for the slave.
 *
 * The slave ComboBox has two additional configuration parameters:
 *
 * - {@link App.form.field.ComboBoxSlave#masterItemId masterItemId}.
 *   It is the itemId of the master widget.
 *   It is required to link the slave and the master widget in order to
 *   implement the logic between them.
 *
 * - {@link App.form.field.ComboBoxSlave#masterValueField masterValueField}.
 *   The field name containing the valueField used by the master.
 *
 * Several slaves can be bind to the same master.
 *
 * @since 0.6.0.0
 *
 */
Ext.define('App.form.field.ComboBoxMaster', {

    extend: 'Ext.form.field.ComboBox',
    alias: 'widget.xcomboboxmaster',
    uses: ['Ext.data.ArrayStore'],

    /**
     * @cfg {Ext.data.Store/String} refStore (required)
     * The reference store shared by the master and by the slave.
     * It contains the relation between the master and slave values.
     * It has to contain the displayField and the valueField for the master
     * and for the slave.
     */
    refStore: null,

    /**
     * @cfg {Boolean} userReset
     * When true, the user can reset the comboBox by selecting
     * the row containing the emptyText.
     *
     */
    userReset: false,

    // Predefined setting
    // Works in auto completion mode when the user type value.
    // In any case the value should belongs to the list.
    editable: true,
    forceSelection: true,
    queryMode: 'local',
    triggerAction: 'all',

    // private properties for internationalization
    textSelect: 'Select a ',

    // private method requires by the ExtJs component model
    initComponent: function () {

        // defined the empty text from the field label
        if (this.fieldLabel && !this.emptyText) {
            this.emptyText = this.textSelect + this.fieldLabel + " ...";
        }

        // instantiate the reference store which contains
        // the relation between the master and the slave value
        this.refStore = App.getStore(this.refStore);

        // Instantiate the local store containing master values.
        // Value are static and almost never change.
        this.store = Ext.create('Ext.data.ArrayStore', {
            fields: [this.displayField, this.valueField]
        });

        // initialise the base class
        this.callParent(arguments);

        // Create the local store from the reference store
        this.createLocalStore();

        // Update the local store when the refStore is modified
        this.refStore.on('remove', this.createLocalStore, this);
        this.refStore.on('update', this.createLocalStore, this);
        this.refStore.on('write', this.createLocalStore, this);

        // user reset
        if (this.userReset) {
            this.on('select', this.onSelect);
        }
    },

    // private method requires by the ExtJs component model
    beforeDestroy: function () {

        this.refStore.un('remove', this.createLocalStore, this);
        this.refStore.un('update', this.createLocalStore, this);
        this.refStore.un('write', this.createLocalStore, this);

        if (this.userReset) {
            this.un('select', this.onSelect);
        }

        this.callParent(arguments);
    },

    /**
     * Create the local store with the value from the reference one
     *
     */
    createLocalStore: function () {

        "use strict";

        if (this.refStore.isFiltered() || !this.refStore.getTotalCount()) {

            this.refStore.restoreWhere();

            this.refStore.load({
                callback: this.onCreate,
                scope: this
            });

        } else {
            this.onCreate(this.refStore.getRange());
        }

        // reset the comboBox
        this.reset();
    },

    /**
     * Handler to create the local store and to load the data
     * from the reference store.
     *
     * @param {Ext.data.Model[]} records
     *
     */
    onCreate: function (records) {

        "use strict";

        var data = [],
            i,
            keys = [],
            master,
            value;

        if (this.userReset) {
            data = [[this.emptyText, null]];
        }

        for (i = 0; i < records.length; i += 1) {

            master = records[i].get(this.displayField);

            if (keys.indexOf(master) === -1) {
                keys.push(master);
                value = records[i].get(this.valueField);
                data.push([master, value]);
            }
        }

        // clear the store and load new data
        this.store.loadData(data, false);
    },

    /**
     * Reset the comBox when the selected row contains the emptyText value.
     *
     * @param {App.form.field.ComboBoxUserReset} combo
     * @param {Array} records
     * @param {Object} eOpts
     */
    onSelect: function (combo, records, eOpts) {

        "use strict";

        // the value is null when the emptyText is selected
        if (!records[0].get(combo.valueField)) {
            combo.setValue("");
        }
    }
});