/**
 * The counter part of the App.form.field.ComboBoxMaster.
 *
 * @since 0.6.0.0
 *
 */
Ext.define('App.form.field.ComboBoxSlave', {

    extend: 'Ext.form.field.ComboBox',
    alias: 'widget.xcomboboxslave',
    uses: ['Ext.data.ArrayStore'],


    /**
     * @cfg {String} masterValueField (required)
     * The name of field containing master values.
     */
    masterValueField: undefined,

    /**
     * @cfg {String} masterItemId (required)
     * The itemId of the master ComboBox.
     */
    masterItemId: undefined,

    /**
     * @cfg {Ext.data.Store/String} refStore (required)
     * The reference store shared by the master and by the slave.
     * It contains the relation between the master and slave values.
     * It has to contain the displayField and the valueField for the master
     * and for the slave.
     */
    refStore: null,

    // Predefined setting
    // Works in auto completion mode when the user type value.
    // In any case the value should belongs to the list.
    editable: true,
    forceSelection: true,
    queryMode: 'local',
    triggerAction: 'all',

    // private properties for internationalization
    textSelect: 'Select a ',

    // private method requires by the ExtJs component model
    initComponent: function () {

        // defined the empty text from the field label
        if (this.fieldLabel && !this.emptyText) {
            this.emptyText = this.textSelect + this.fieldLabel + " ...";
        }

        // instantiate the reference store which contains
        // the relation between the master and the slave value
        this.refStore = App.getStore(this.refStore);

        // build the local store containing slave and master values
        this.store = Ext.create('Ext.data.ArrayStore', {
            fields: [
                this.masterValueField,
                this.displayField,
                this.valueField
            ]
        });

        // initialise the base class
        this.callParent(arguments);

        // create the local store from the reference store
        this.createLocalStore();

        // Bind the slave to the master widget.
        // Can only be done when widgets exists.
        this.on('afterrender', this.bindToMaster, this, {single: true});

        // update the local store is the refStore is modified
        this.refStore.on('remove', this.createLocalStore, this);
        this.refStore.on('update', this.createLocalStore, this);
        this.refStore.on('write', this.createLocalStore, this);
    },

    // private method requires by the ExtJS component model
    beforeDestroy: function () {

        this.refStore.un('remove', this.createLocalStore, this);
        this.refStore.un('update', this.createLocalStore, this);
        this.refStore.un('write', this.createLocalStore, this);

        this.callParent(arguments);
    },

    /**
     * Bind the master and the slave comboBoxes.
     * The handler is triggered by the afterrender event.
     *
     */
    bindToMaster: function () {

        "use strict";

        // Find the master widget
        // Assume that it is above in the widget stack.
        // Not the optimal way but failed to use this.up(path) ???
        var path = 'xcomboboxmaster[itemId=' + this.masterItemId + ']',
            master = this.getRefOwner().query(path)[0],
            masterValue = master.getValue();

        master.on('change', this.onMasterChange, this);

        // initialize the comboBox.
        this.store.clearFilter(true);

        if (masterValue === null || masterValue === undefined || masterValue === "") {
            this.setValue("");

        } else {
            this.store.filter(this.masterValueField, masterValue);
        }
    },

    /**
     * Create the the local store and fill it
     * with the value from the reference one.
     *
     */
    createLocalStore: function () {

        "use strict";

        if (this.refStore.isFiltered() || !this.refStore.getTotalCount()) {

            this.refStore.restoreWhere();

            this.refStore.load({
                callback: this.onCreate,
                scope: this
            });

        } else {
            this.onCreate(this.refStore.getRange());
        }
    },

    /**
     * Handler to load the slave data into the local store
     * from the reference store to the local store.
     * The pair of values (master, slave) is unique.
     *
     * @param {Ext.data.Model[]} records
     *
     */
    onCreate: function (records) {

        "use strict";

        var data = [],
            i,
            key,
            keys = [],
            master,
            slave,
            value;

        for (i = 0; i < records.length; i += 1) {

            master = records[i].get(this.masterValueField);
            slave = records[i].get(this.displayField);

            key = master + slave;

            if (keys.indexOf(key) === -1) {
                keys.push(key);
                value = records[i].get(this.valueField);
                data.push([master, slave, value]);
            }
        }

        // clear the store and load new data
        this.store.loadData(data, false);
    },

    /**
     * Select the authorized values for the slave values and
     * load the first one.
     *
     * @param {Ext.form.field.ComboBox} combo
     * @param {Ext.data.Model[]} records
     * @param {Object} eOpts
     */
    onMasterChange: function (combo, newValue, oldValue) {

        "use strict";

        var masterValue = newValue;

        this.store.clearFilter(true);

        if (masterValue === null || masterValue === undefined || masterValue === "") {
            this.setValue("");

        } else {
            this.store.filter(this.masterValueField, masterValue);
            if (this.store.getCount()) {
                this.setValue(this.store.first().get(this.valueField));
            } else {
                this.setValue("");
            }
        }
    },

    /**
     * Set the value of the slave ComboBox.
     * The field is disabled when the value is empty.
     *
     * @param {String/String[]} value
     * The value to be set.
     *
     * @return {Ext.form.Field} this
     *
     */
    setValue: function (value) {

        if (value === "" || value === null || value === undefined) {
            this.setDisabled(true);

        } else {
            this.setDisabled(false);
        }

        return this.callParent(arguments);
    }
});