/**
 * The form field to render the web2py database type JSON.
 * The JSON object is a dictionary. It can only contains boolean,
 * date, float and string objects.
 *
 * The default structure of the dictionary can be defined via
 * the configuration option #value:
 *     {
 *         "(name)": "My Object",
 *         "Created": new Date(Date.parse('10/15/2006')),
 *         "Available": false,
 *         "Version": .01,
 *         "Description": "A test object"
 *     }
 *
 * The basic syntax is also working:
 *     {
 *          foo: 1.23
 *     }
 *
 * Keys can be added / modified / deleted key when the configuration
 * option #modifyKeys is true.
 *
 * The interface is less rich than the one of the Ext.form.field.Base.
 * It contains the basic methods getValue, setValue, reset, ...
 *
 * @since 0.4.15.0
 *
 */
Ext.define('App.form.field.Dict', {

    extend: 'Ext.form.FieldContainer',
    mixins: {
        field: 'Ext.form.field.Field'
    },
    alias: 'widget.xdictfield',
    uses: ['Ext.grid.property.Grid', 'Ext.menu.Menu'],

    /**
     * @cfg {String[]}
     * The labels of the columns to be shown in the grid widget.
     */
    headers: null,

    /**
     * @cfg {Boolean}
     * The columns header are hidden when true.
     */
    hideHeaders: false,

    /**
     * @cfg {Boolean}
     * Dictionary keys can be added or deleted when true.
     */
    modifyKeys: false,

    /**
     * @event keychange
     * Fire when a key is added, modified or destroyed.
     *
     * @param {object} this
     * @param {String} action
     * @param {String} oldKey
     * @param {String} newKey
     */

    // Predefined configuration options
    layout: 'anchor',
    defaults: {
        anchor: '100%'
    },

    // private short cuts
    grid: null,
    menu: null,

    // private properties for internationalization
    textAddMenu: 'Add',
    textAddTitle: 'Please enter the new key',
    textDestroyMenu: 'Delete',
    textDestroyTitle: 'Are you sure to destroy the key',
    textErrorTitle: 'Error ...',
    textErrorMsg: 'The key already exists!',
    textUpdateMenu: 'Update',
    textUpdateTitle: 'Modify the key',

    // private method requests by the component model of ExtJS
    initComponent: function () {

        var me = this,
            columns,
            i,
            headerContainer,
            source = me.value || {};

        // configure the property grid embedded in the container
        me.items = [{
            xtype: 'propertygrid',
            hideHeaders: me.hideHeaders,
            source: source
        }];

        // instantiate the base class
        me.callParent(arguments);

        // short cut to the grid
        me.grid = me.child('propertygrid');

        // dedicated column headers
        if (me.headers) {

            headerContainer = me.grid.getDockedItems('headercontainer')[0];
            columns = headerContainer.getGridColumns();

            for (i = 0; i < columns.length; i += 1) {
                columns[i].setText(me.headers[i]);
            }
        }

        // context menu
        me.grid.on('cellcontextmenu', me.onCellContextMenu, me);
        me.grid.on('containercontextmenu', me.onContainerContextMenu, me);
        me.grid.on('headercontextmenu', me.onHeaderContextMenu, me);

        if (me.modifyKeys) {

            me.menu = Ext.create('Ext.menu.Menu', {
                items: [{
                    text: me.textAddMenu,
                    iconCls: 'xaction-create',
                    itemId: 'add',
                    handler: me.onAddKey,
                    scope: me
                }, '-', {
                    text: me.textUpdateMenu,
                    iconCls: 'xaction-update',
                    itemId: 'update',
                    handler: me.onUpdateKey,
                    scope: me
                }, '-', {
                    text: me.textDestroyMenu,
                    iconCls: 'xaction-destroy',
                    itemId: 'destroy',
                    handler: me.onDeleteKey,
                    scope: me
                }]
            });
        }

    // initialize the mixin Field
    me.initField();
    },

    // private method requests by the component model of ExtJS
    beforeDestroy: function () {

        var me = this;

        me.grid.un('cellcontextmenu', me.onCellContextMenu, me);
        me.grid.un('containercontextmenu', me.onContainerContextMenu, me);
        me.grid.un('headercontextmenu', me.onHeaderContextMenu, me);

        Ext.destroyMembers(me, 'grid');
        Ext.destroyMembers(me, 'menu');

        me.callParent(arguments);
    },

    /**
     * Return the value of me field.
     *
     * @return {Object} the content of the dictionary.
     *
     */
    getValue: function () {

        "use strict";
        var me = this;

        return Ext.apply({}, me.grid.getSource());
    },

    /**
     * Add a key to the dictionary.
     */
    onAddKey: function () {

        "use strict";

        var me = this,
            source = me.grid.getSource();

        // prompt the user to get the key name
        Ext.Msg.prompt(me.textAddTitle, "", function (btn, text) {

            if (btn === 'ok') {

                // check that the key is unique
                if (source.hasOwnProperty(text)) {
                    Ext.Msg.alert(me.textErrorTitle, me.textErrorMsg);
                    return;
                }
                // add the new key
                source[text] = null;
                me.grid.setSource(source);

                me.fireEvent('keychange', me, 'add', undefined, text);
            }
        }, me);
    },

    /**
     * Show the context menu to modify key when right clicking on the first row.
     *
     * @param {Ext.view.Table} table
     * @param {HTMLElement} td
     * @param {Number} cellIndec
     * @param {Ext.data.Model} record
     * @param {HTMLElement} tr
     * @param {Number} rowIndex
     * @param {Ext.EventObject} event
     * @param {Object} eOpts
     *
     */
    onCellContextMenu: function (table, td, cellIndex, record, tr, rowIndex, event, eOpts) {

        "use strict";
        var me = this;

        event.stopEvent();

        if (me.modifyKeys && cellIndex === 0) {

            me.menu.getComponent('destroy').setDisabled(false);
            me.menu.getComponent('update').setDisabled(false);

            me.menu.showAt(event.getXY());
        }
    },

    /**
     * Show the context menu when the grid is empty.
     * In that case only the add option is allowed.
     *
     * @param {Ext.view.View} view
     * @param {Ext.EventObject} event
     * @param {Object} eOpts
     *
     */
    onContainerContextMenu: function (view, event, eOpts) {

        "use strict";
        var me = this;

        event.stopEvent();

        if (me.modifyKeys) {
            me.menu.getComponent('destroy').setDisabled(true);
            me.menu.getComponent('update').setDisabled(true);

            me.menu.showAt(event.getXY());
        }
    },

    /**
     * Delete a key of the dictionary.
     */
    onDeleteKey: function () {

        "use strict";

        var me = this,
            record = me.grid.getSelectionModel().getSelection()[0],
            prop = record.get("name");

        Ext.Msg.confirm(me.textDestroyTitle, prop, function (btn) {
            if (btn === "yes") {
                me.grid.removeProperty(prop);
                me.fireEvent('keychange', me, 'destroy', prop, undefined);
            }
        }, me);
    },

    /**
     * Inhibit the context menu when the user right click on the header
     *
     * @param {Ext.grid.header.Container} headerContainer
     * @param {Ext.grid.column.Column} column
     * @param {Ext.EventObject} event
     * @param {HTMLElement} html
     * @param {Object} eOpts
     */
    onHeaderContextMenu: function (headerContainer, column, event, html, eOpts) {

        "use strict";

        var me = this,
            source = me.grid.getSource(),
            isSourceEmpty = Object.keys(source).length === 0;

        event.stopEvent();

        // allow to add key if source is not defined
        if (me.modifyKeys && isSourceEmpty && column.getItemId() === 'name') {
            me.onAddKey();
        }
    },

    /**
     * Update a key.
     */
    onUpdateKey: function () {

        "use strict";

        var me = this,
            record = me.grid.getSelectionModel().getSelection()[0],
            key = record.get("name"),
            value = record.get("value"),
            source = me.grid.getSource();

        // prompt the user to modify the key
        Ext.Msg.prompt(me.textUpdateTitle, "", function (btn, text) {

            if (btn === 'ok') {

                // check that the key is unique
                if (source.hasOwnProperty(text)) {
                    Ext.Msg.alert(me.textErrorTitle, me.textErrorMsg);
                    return;
                }
                // update the key
                delete source[key];
                source[text] = value;
                me.grid.setSource(source);

                me.fireEvent('keychange', me, 'update', key, text);
            }
        }, me, false, key);

    },

    /**
     * Set the value for the dictionary.
     * Load the default dictionary when the value is null.
     *
     * @param {Object} value
     * A key, value object which is loaded in the configuration grid.
     *
     */
    setValue: function (value) {

        "use strict";

        var me = this,
            source  = value || me.originalValue;

        me.grid.setSource(source);

        // the standard way to set the value for a field
        // extract from Ext.form.field.Base
        return me.mixins.field.setValue.call(me, source);
    }
});
