/**
 * The form field to render the web2py database type list:string.
 *
 * The interface is less rich than the one of the Ext.form.field.Base.
 * It contains the basic methods getValue, setValue, reset, ...
 *
 * @since 0.4.15.0
 *
 */
Ext.define('App.form.field.List', {

    extend: 'Ext.form.FieldContainer',
    mixins: {
        field: 'Ext.form.field.Field'
    },
    alias: 'widget.xlistfield',
    uses: ['Ext.data.ArrayStore',
           'Ext.form.field.Text',
           'Ext.grid.Panel',
           'Ext.grid.plugin.CellEditing',
           'Ext.menu.Menu'],
    /**
     * @cfg {string}
     * The label of the column to be shown in the grid widget.
     */
    header: 'Element',

    /**
     * @cfg {boolean}
     * Hide the column header when true.
     */
    hideHeader: false,

    /**
     * @cfg{boolean}
     * Empty string are not copied in the list return by the #getValue method.
     */
    ignoreEmptyRows: true,

    /**
     * @cfg {number}
     * The minimum number of element displayed in the list.
     * The grid is filled with empty string when values are missing.
     * Empty string can be ignored when getting the list using
     * the property #ignoreEmptyRows.
     *
     */
    minimumRows: 3,

    // Predefined configuration options
    layout: 'anchor',
    defaults: {
        anchor: '100%'
    },

    // private short cuts
    cellEditing: null,
    grid: null,
    menu: null,
    record: null,
    rowIndex: undefined,
    store: null,

    // private properties for internationalization
    textAdd: 'Insert element',
    textDestroy: 'Delete element',
    textDestroyTitle: 'Are you sure to destroy the element',
    textGoDown: 'Move down',
    textGoUp: 'Move up',

    // private method requests by the component model of ExtJS
    initComponent: function () {

        var me = this,
            data = [],
            grid,
            i;

        // instantiate the local store
        // it is filled with dummy data
        for (i = 0; i < me.minimumRows; i += 1) {
            data.push([]);
        }

        me.store = Ext.create('Ext.data.ArrayStore', {
            fields: [{name: 'element'}],
            data: data
        });

        // configure the grid embedded in the container
        // it will use the cellEditing plugin
        // when a cell is edited it is not marked dirty
        me.items = [{
            columns: [{
                dataIndex: 'element',
                editor: Ext.create('Ext.form.field.Text', {}),
                flex: 1,
                itemId: 'element',
                sortable: false,
                text: me.header
            }],
            hideHeaders: me.hideHeader,
            plugins: {
                ptype: 'cellediting',
                triggerEvent: 'cellclick'
            },
            selType: 'cellmodel',
            store: me.store,
            viewConfig: {
                markDirty: false
            },
            xtype: 'grid'
        }];

        me.callParent(arguments);

        // instantiate the context menu
        me.menu = Ext.create('Ext.menu.Menu', {
            items: [{
                text: me.textAdd,
                iconCls: 'xaction-create',
                itemId: 'add',
                handler: me.onAddRow,
                scope: me
            }, '-', {
                text: me.textGoUp,
                iconCls: 'xaction-go-up',
                itemId: 'goUp',
                handler: me.onGoUp,
                scope: me
            }, {
                text: me.textGoDown,
                iconCls: 'xaction-go-down',
                itemId: 'goDown',
                handler: me.onGoDown,
                scope: me
            }, '-', {
                text: me.textDestroy,
                iconCls: 'xaction-destroy',
                itemId: 'destroy',
                handler: me.onDeleteRow,
                scope: me
            }]
        });

        // handler associate to the grid
        me.grid = me.child('grid');

        me.grid.on('containercontextmenu', me.onContainerContextMenu, me);
        me.grid.on('itemcontextmenu', me.onItemContextMenu, me);

        // avoid editing when the menu is visible
        me.cellEditing = me.grid.findPlugin('cellediting');
        me.cellEditing.on('beforeedit', me.onBeforeEdit, me);

        // initialize the mixin Field
        me.initField();
    },

    // private method requests by the component model of ExtJS
    beforeDestroy: function () {

        var me = this;

        me.cellEditing.un('beforeedit', me.onBeforeEdit, me);

        me.grid.un('containercontextmenu', me.onContainerContextMenu, me);
        me.grid.un('itemcontextmenu', me.onItemContextMenu, me);

        Ext.destroyMembers(me, 'grid');
        Ext.destroyMembers(me, 'menu');
        Ext.destroyMembers(me, 'store');

        me.callParent(arguments);
    },

    /**
     * Return the value of me field
     * Empty string are not copied in the list when #ignoreEmptyRows is true.
     *
     * @return {Array}
     *
     */
    getValue: function () {

        "use strict";

        var me = this,
            i,
            li = [],
            records = me.store.getRange(),
            value;

        for (i = 0; i < records.length; i += 1) {

            value = records[i].get("element");

            if (!me.ignoreEmptyRows || value) {
                li.push(value);
            }
        }

        return li;
    },

    /**
     * Add an element to the list below the selected one.
     */
    onAddRow: function () {

        "use strict";

        var me = this,
            record = me.store.getProxy().getModel().create();

        record.set("element", "");
        me.store.insert(me.rowIndex + 1, record);
    },

    /**
     * Inhibit the cell editing if the menu is visible.
     * me is required since the cell editing is trigger by any
     * cell click (left or right mouse button).
     *
     */
    onBeforeEdit: function () {

        "use strict";
        var me = this;

        if(me.menu.isVisible()) {
            return false;
        }
    },

    /**
     * Show the context menu.
     *
     * @param {Ext.view.View} view
     * @param {Ext.EventObject} event
     * @param {Object} eOpts
     *
     */
    onContainerContextMenu: function (view, event, eOpts) {

        "use strict";
        var me = this;

        event.stopEvent();

        me.menu.getComponent('destroy').setDisabled(true);
        me.menu.getComponent('goDown').setDisabled(true);
        me.menu.getComponent('goUp').setDisabled(true);

        me.rowIndex = -1;
        me.menu.showAt(event.getXY());
    },

    /**
     * Delete the selected element of the list.
     */
    onDeleteRow: function () {

        "use strict";

        var me = this,
            value = me.record.get("element");

        Ext.Msg.confirm(me.textDestroyTitle, value, function (btn) {
            if (btn === 'yes') {
                me.store.remove(me.record);
            }
        }, me);
    },

    /**
     * Move the selected element down.
     */
    onGoDown: function () {

        "use strict";

        var me = this,
            recordA,
            recordB,
            valueA,
            valueB;

        recordA = me.store.getAt(me.rowIndex);
        recordB = me.store.getAt(me.rowIndex + 1);

        valueA = recordA.get('element');
        valueB = recordB.get('element');

        recordA.set('element', valueB);
        recordB.set('element', valueA);
    },

    /**
     * Move the selected element up.
     */
    onGoUp: function () {

        "use strict";

        var me = this,
            recordA,
            recordB,
            valueA,
            valueB;

        recordA = me.store.getAt(me.rowIndex);
        recordB = me.store.getAt(me.rowIndex - 1);

        valueA = recordA.get('element');
        valueB = recordB.get('element');

        recordA.set('element', valueB);
        recordB.set('element', valueA);
    },

    /**
     * Show the row context menu.
     *
     * @param {Ext.view.View} view
     * @param {Ext.data.Model} record
     * @param {HTMLElement} item
     * @param {Number} index
     * @param {Ext.EventObject} event
     * @param {Object} eOpts
     *
     */
    onItemContextMenu: function (view, record, item, index, event, eOpts) {

        "use strict";
        var me = this;

        event.stopEvent();

        // protection to hide the menu if it is already visible
        if (me.menu.isVisible()) {
            me.menu.hide();
            return;
        }

        me.record = record;
        me.rowIndex = index;

        me.menu.showAt(event.getXY());

        me.menu.getComponent('destroy').setDisabled(false);
        me.menu.getComponent('goDown').setDisabled(false);
        me.menu.getComponent('goUp').setDisabled(false);

        if (index === 0) {
            me.menu.getComponent('goUp').setDisabled(true);
        }

        if (index === me.store.getCount() - 1) {
            me.menu.getComponent('goDown').setDisabled(true);
        }

    },

    /**
     * Set the value for me field.
     *
     * @param {Array} value
     * The content of the list.
     */
    setValue: function (value) {

        "use strict";

        var me = this,
            data = [],
            i;

        // convert the list in a list of list
        if (value && value instanceof Array) {

            for (i = 0; i < value.length; i += 1) {
                data.push([value[i]]);
            }

            if (value.length < me.minimumRows) {
                for (i = value.length; i < me.minimumRows; i += 1) {
                    data.push([]);
                }
            }

        } else {

            for (i = 0; i < me.minimumRows; i += 1) {
                data.push([]);
            }
        }

        // replace the content of the store with the new data set
        me.store.loadData(data, false);

        // the standard way to set the value for a field
        // extract from Ext.form.field.Base
        return me.mixins.field.setValue.call(me, value);
    }
});