/**
 * Special field well suited for grid cell editing.
 *
 * It appears as a standard text field but behave as a TextArea.
 * The latter is encapsulated in a pop up appearing when clicking
 * on the text field.
 *
 * The value displayed in the text field is an ellipsis version of
 * the content of the TextArea.
 *
 * @experimental
 * @since 0.6.2.4
 *
 */
Ext.define('App.form.field.TextPicker', {

    extend: 'Ext.form.field.Picker',
    alias: 'widget.xtextpicker',
    uses: ['Ext.form.field.TextArea'],

    /**
     * @cfg {Number}
     * The maximum length to allow before truncation and add an ellipsis
     */
    ellipsis: 10,

    /**
     * @cfg {Number}
     * Initial number of rows displayed in the text area.
     */
    rows: 4,

    // Predefined configuration options
    editable: false,
    hideTrigger: true,

    /**
     * Create and return the TextArea encapsulated in the picker.
     *
     * @return {Ext.form.field.TextArea}
     *
     */
    createPicker: function() {

        "use strict";

        var me = this,
            cfg,
            picker;

        cfg = {
            floating: true,
            rows: me.rows,
            shadow: false,
            xtype: 'textareafield'
        };

        picker = me.picker = Ext.widget(cfg);
        return picker;
    },

    /**
     * Handler execute when the picker is collapsed.
     * It set up the value displayed in the text field from the
     * content of the text area.
     *
     */
    onCollapse: function() {

        "use strict";

        var me = this,
            value;

        value = me.picker.getValue();
        me.setDisplayValue(value);
    },

    /**
     * Return the content of the text area.
     */
    getValue: function() {

        "use strict";
        var me = this;

        return me.getPicker().getValue();
    },

    // Set the value displayed in the text field.
    // It it the ellipsis version of value, removing line break.
    setDisplayValue: function(value) {

        "use strict";
        var me = this;

        value = value.replace("\n", "");
        value = Ext.String.ellipsis(value, me.ellipsis, true);

        me.inputEl.dom.value = value;

    },

    /**
     * Set the content of the text area and of the text field.
     *
     * @param {Object} value
     */
    setValue: function(value) {

        "use strict";
        var me = this;

        if (value) {
            me.getPicker().setValue(value);
            me.setDisplayValue(value);
        }

        return me;
    }
});
