/**
 * The plugin renders mathematics equation embedded in panel and grid.
 * The processing is performed by [MathJax](http://www.mathjax.org/).
 * The MathJax library is loaded by the framework.
 *
 * The plugin works with the classes: Ext.panel.Panel, Ext.grid.GridPanel,
 * App.grid.GridPanel, App.panel.BaseWithSelector, App.panel.WithUrlSelector,
 * and App.panel.GridWithFilter.
 *
 * @since 0.6.0.10
 *
 */
Ext.define('App.plugin.MathJax', {

    extend: 'Ext.AbstractPlugin',
    alias: 'plugin.pMathJax',
    uses: ['Ext.LoadMask'],

    /**
     * @property {Boolean}
     * False to deactivate the MathJax processing.
     */
    activated: true,

    // Private short cuts
    target: null,
    targetDomEl: null,
    targetType: undefined,

    // Private attributes for internationalization
    textMask: 'Processing symbols',

    /**
     * Initialize the plugin
     *
     * @param {Ext.grid.GridPanel/Ext.panel.Panel/App.panel.BaseWithSelector} cmp
     *
     */
    init: function (cmp) {

        "use strict";
        var xtype = cmp.getXType();

        // protection
        if (!MathJax) {
            return;
        }

        this.setCmp(cmp);

        // determine the target component to be processed
        switch (xtype) {
        case 'grid':
            this.target = cmp.getView();
            this.targetType = 'grid';
            break;

        case 'panel':
            this.target = cmp;
            this.targetType = 'panel';
            break;

        case 'xpanelwithurlselector':
            this.target = cmp.mainPanel;
            this.targetType = 'panel';
            break;

        case 'xpanelwithselector':
            this.target = cmp.mainPanel;
            this.targetType = 'panel';
            break;

        case 'xgrid':
            this.target = cmp.getView();
            this.targetType = 'grid';
            break;

        case 'xgridwithfilter':
            this.target = cmp.grid.getView();
            this.targetType = 'grid';
            break;
        }

        // the DOM element of the target
        this.targetDomEl = Ext.getDom(this.target.getEl());

        // listener to trigger the MathJax processing
        switch (this.targetType) {
        case 'grid':
            //
            // process equation each time the grid is refreshed.
            //
            // NOTE: MathJax processing is lost when the gridView is
            // modified (sort, filter, add, remove or delete).
            //
            this.target.on('refresh', this.onProcessMath, this);
            break;

        case 'panel':
            //
            // process equation when the panel is loaded
            //
            this.target.getLoader().on('load', this.onProcessMath, this);
            break;
        }
    },

    /**
     * destroy the plugin
     */
    destroy: function () {

        "use strict";
        switch (this.targetType) {

        case 'grid':
            this.target.un('refresh', this.onProcessMath, this);
            break;

        case 'panel':
            this.target.getLoader().un('load', this.onProcessMath, this);
            break;
        }
    },

    /**
     * Process maths objects
     *
     */
    onProcessMath: function () {

        "use strict";

        var cmp = this.getCmp(),
            loadMask,
            parent,
            targetMask = cmp;

        if (!this.activated) {
            return;
        }

        // parent of a grid can be gridWithFilter.
        // In that case the gridWithFilter should be masked
        //
        // NOTE: the parent is not define when the plugin is initialized.
        //
        if (this.targetType === 'grid') {
            parent = cmp.up('xgridwithfilter');
            if (parent) {
                targetMask = parent;
            }
        }

        // mask the component
        loadMask = Ext.create('Ext.LoadMask', {
            target: targetMask,
            msg: this.textMask
        });

        loadMask.show();

        // MatJax processing followed by the unmasking
        MathJax.Hub.Queue(["Typeset", MathJax.Hub, this.targetDomEl]);
        MathJax.Hub.Queue([function (mask) {mask.hide(); }, loadMask]);
    }
});